<?php
/**
 * Main content controller; handles both frontend and administrative utilities.
 *
 * @package Content
 * @since 0.1
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\AccessDeniedException;
use Core\Controller;
use Core\Forms\Form;
use Core\NotFoundException;
use Core\TemporaryFollowRedirectException;
use Core\ViewControl;


class ContentController extends Controller {
	public function view(){
		// I'm calling check access here because the cached access string is canonical in this case.
		$request = $this->getRequest();
		$page    = $this->getPageModel();
		$view    = $this->getView();

		if(!\Core\user()->checkAccess($page->get('access'))){
			throw new AccessDeniedException();
		}

		/** @var $m ContentModel */
		$m = ContentModel::Construct($request->getParameter(0));
		
		if(!$m->checkExists()) {
			throw new NotFoundException();
		}

		$editor  = (\Core\user()->checkAccess($m->get('editpermissions')) || \Core\user()->checkAccess('p:/content/manage_all'));
		$manager = \Core\user()->checkAccess('p:/content/manage_all');

		if(!$page->isPublished()){
			if($editor){
				\Core\set_message('This page is currently set as ' . $page->getPublishedStatus() . '.', 'info');
			}
			else{
				throw new NotFoundException();
			}
		}

		$view->assign('model', $m);
	    $view->assign('page', $page);
		//$view->templatename = '/pages/content/' . $template;
		//$view->updated = $m->get('updated');

		if($manager){
			$view->addControl(ViewControl::FactoryArray([
				'title' => 't:STRING_ADD_PAGE',
				'link' => '/content/create?page_template=' . $page->get('base_template') . '&parenturl=' . $page->get('baseurl'),
				'icon' => 'add',
				'class' => 'add',
			]));
		}
		if($editor){
			$view->addControl(ViewControl::FactoryArray([
				'title' => 't:STRING_EDIT_PAGE',
				'link' => '/content/edit/' . $m->get('id'),
				'icon' => 'edit',
				'class' => 'edit',
			]));
		}
		if($manager){
			$view->addControl(ViewControl::FactoryArray([
				'title' => 't:STRING_DELETE_PAGE',
				'link' => '/content/delete/' . $m->get('id'),
				'icon' => 'delete',
				'confirm' => t('MESSAGE_DELETE_PAGE'),
			]));
		}
	}

	/**
	 * Edit an existing page
	 * 
	 * This has dynamic permissions, so access must be checked manually
	 * 
	 * @throws NotFoundException
	 * @throws AccessDeniedException
	 */
	public function edit(){
		$view    = $this->getView();
		$request = $this->getRequest();
		$model   = new ContentModel($request->getParameter(0));
		$page    = $model->getLink('Page');
		

		if(!$model->checkExists()){
			throw new NotFoundException();
		}

		$editor  = (\Core\user()->checkAccess($model->get('editpermissions')) || \Core\user()->checkAccess('p:/content/manage_all'));
		$manager = \Core\user()->checkAccess('p:/content/manage_all');

		if(!($editor || $manager)){
			throw new AccessDeniedException();
		}
		
		$form = new Form();
		$form->set('callsmethod', 'ContentController::_SaveHandler');

		$form->addModel($page, 'page');
		$form->addModel($model, 'model');

		// Tack on a submit button
		$form->addElement('submit', array('value' => 't:STRING_UPDATE'));

		// Editors have certain permissions here, namely limited.
		if(!$manager){
			$form->removeElement('model[nickname]');
			$form->removeElement('model[editpermissions]');
			$form->removeElement('page[rewriteurl]');
			$form->removeElement('page[parenturl]');
		}
		
		$pageTitle = $page->get('title') ?: t('STRING_NO_PAGE_TITLE');
		
		$view->mastertemplate = 'admin';
		$view->addBreadcrumb($pageTitle, $page->get('baseurl'));
		$view->title        = 'Edit ' . $pageTitle;
		$view->assignVariable('model', $model);
		$view->assignVariable('form', $form);
	}

	/**
	 * Create a new content page on the site
	 * 
	 * @core-access-string p:/content/manage_all
	 */
	public function create() {

		$view = $this->getView();
		$request = $this->getRequest();

		$model = new ContentModel();
		$page = $model->getLink('Page');
		
		// Allow the user to specify a parent URL to default to.
		// This is used with the "add page 'here'" option.
		if($request->getParameter('parenturl')){
			$page->set('parenturl', $request->getParameter('parenturl'));
		}
		if($request->getParameter('page_template')){
			$page->set('page_template', $request->getParameter('page_template'));
		}

		$form = new Form();
		$form->set('callsmethod', 'ContentController::_SaveHandler');

		$form->addModel($page, 'page');
		$form->addModel($model, 'model');
		
		// Tack on a submit button
		$form->addElement('submit', array('value' => 't:STRING_CREATE'));


		$view->mastertemplate = 'admin';
		$view->templatename = '/pages/content/create.tpl';
		$view->title        = 't:STRING_NEW_CONTENT_PAGE';
		$view->assignVariable('model', $model);
		$view->assignVariable('form', $form);
	}

	public static function _SaveHandler(Form $form) {

		/** @var $model ContentModel */
		$model = $form->getModel('model');
		/** @var $page PageModel Page object for this model, already linked up! */
		$page = $form->getModel('page');

		$isnew = $model->checkIsNew();

		// The content nickname is derived from the page title.
		$model->set('nickname', $page->get('title'));
		if($page->checkExists() && $page->checkIsChanged('site')){
			$model->set('site', $page->get('site'));
		}
		$model->save();

		$page->set('editurl', '/content/edit/' . $model->get('id'));
		$page->set('deleteurl', '/content/delete/' . $model->get('id'));
		$page->set('component', 'content');
		// Force this page to be updated at the time of updating.
		$page->set('updated', \Core\Date\DateTime::NowGMT());
		$page->save();

		// Clear the page cache
		$page->purgePageCache();

		// w00t
		$msg = ($isnew ? 'Added' : 'Updated');
		$link = \Core\resolve_link($page->get('baseurl'));
		\Core\set_message('<a href="' . $link . '">' . $msg . ' page successfully!</a>', 'success');
		return 'back';
	}

	/**
	 * @core-request-method POST
	 * @core-accessstring p:/content/manage_all
	 */
	public function delete() {
		$view    = $this->getView();
		$request = $this->getRequest();

		$m    = new ContentModel($request->getParameter(0));
		$link = \Core\resolve_link($m->get('baseurl'));

		if (!$m->exists()){
			throw new NotFoundException();
		}
		$m->delete();

		\Core\set_message('Removed ' . $m->get('nickname') . ' successfully!', 'success');

		$hist = $request->getReferrer();
		if($hist == $link){
			throw new TemporaryFollowRedirectException('/admin/pages');
		}
		else{
			throw new TemporaryFollowRedirectException('BACK');
		}
	}
}

